#!/usr/bin/python2

import os
import time
import errno
import subprocess
import multiprocessing as mp

class ProcessManager:

    manager = mp.Manager()
    processes = dict()
    subprocesses = manager.dict()
    results = manager.dict()

    def cleanupProcesses(self):
        self.processes.clear()
        self.subprocesses.clear()
        self.results.clear()

    def runProcess(self,
                   processKey,
                   processCallFormat,
                   message,
                   subprocesses,
                   results,
                   inputFile,
                   outputFile):
        print message
        stdin = None
        if inputFile is not None:
            stdin = open(inputFile, "r")
        stdout = subprocess.PIPE
        if outputFile is not None:
            stdout = open(outputFile, "w")
        process = subprocess.Popen(
            processCallFormat,
            stdin=stdin,
            stdout=stdout,
            stderr=subprocess.PIPE)
        subprocesses[processKey] = process
        try:
            stdout, stderr = process.communicate()
            returnCode = process.returncode
            results[processKey] = (returnCode, stdout, stderr)
        except IOError as e:
            print e
            pass

    def startProcess(self,
                     processKey,
                     processCallFormat,
                     message,
                     inputFile=None,
                     outputFile=None):
        self.processes[processKey] = mp.Process(
            target=self.runProcess,
            args=[processKey,
                  processCallFormat,
                  message,
                  self.subprocesses,
                  self.results,
                  inputFile,
                  outputFile])
        self.processes[processKey].start()

    def killProcess(self, processKey):
        if processKey not in self.results and processKey in self.subprocesses:
            self.subprocesses[processKey].terminate()

    def hasProcessCompleted(self, processKey):
        if processKey in self.results:
            return True
        else:
            return False

    def waitForProcessCompletion(self, processKey, waitTime):
        self.processes[processKey].join(waitTime)

    def getProcessReturnCode(self, processKey):
        if processKey in self.results:
            (returnCode, stdout, stderr) = self.results[processKey]
            return returnCode
        else:
            print "Invalid processKey provided - " + processKey
            return -1

    def getProcessError(self, processKey):
        if processKey in self.results:
            (returnCode, stdout, stderr) = self.results[processKey]
            return stderr
        else:
            return "Error not available for processKey - " + processKey

    def getProcessOutput(self, processKey):
        if processKey in self.results:
            (returnCode, stdout, stderr) = self.results[processKey]
            return stdout
        else:
            return "Output not available for processKey - " + processKey

    def startNfd(self):
        self.startProcess("nfd", ["sudo", "nfd"], "-> Starting NFD")

    def killNfd(self):
        self.killProcess("nfd")
        time.sleep(2)

def checkServerStatus(p, processKey, processName):
    serverOutput = p.getProcessOutput(processKey).splitlines()
    isServerSuccessful = True
    for logLine in serverOutput:
        if "Prefix Registration Failed" in logLine:
            isServerSuccessful = False
            break
    if not isServerSuccessful:
        print ">> SERVER FAILURE - " + processName
        print logLine


def main():
    p = ProcessManager()

    #Start NFD
    p.startNfd()
    time.sleep(1)

    #Start ndn-traffic-server to serve ndn:/ with Content "Z", FreshnessPeriod=10ms
    #Prefix registration using NFD FibMgmt
    serverConfigurationFile = os.path.abspath("test-traffic-server-1.conf")
    p.startProcess("ndn-traffic-server-1",
        ["ndn-traffic-server", serverConfigurationFile],
        "-> Starting Traffic Server 1 (Prefix=ndn:/, FreshnessPeriod=10ms, Data=Z)")
    #time.sleep(0.1)

    #Start ndn-traffic-server to serve ndn:/A with Content "A", FreshnessPeriod=10ms
    #Prefix registration using NFD FibMgmt
    serverConfigurationFile = os.path.abspath("test-traffic-server-2.conf")
    p.startProcess("ndn-traffic-server-2",
        ["ndn-traffic-server", serverConfigurationFile],
        "-> Starting Traffic Server 2 (Prefix=ndn:/A, FreshnessPeriod=10ms, Data=A)")
    #time.sleep(0.1)

    #Start ndn-traffic-server to serve ndn:/A/B/C with Content "C", FreshnessPeriod=10ms
    #Prefix registration using NFD FibMgmt
    serverConfigurationFile = os.path.abspath("test-traffic-server-3.conf")
    p.startProcess("ndn-traffic-server-3",
        ["ndn-traffic-server", serverConfigurationFile],
        "-> Starting Traffic Server 3 (Prefix=ndn:/A/B/C, FreshnessPeriod=10ms, Data=C)")
    #time.sleep(0.1)

    #Start ndn-traffic-server to serve ndn:/D/E with Content "E", FreshnessPeriod=10ms
    #Prefix registration using NFD FibMgmt
    serverConfigurationFile = os.path.abspath("test-traffic-server-4.conf")
    p.startProcess("ndn-traffic-server-4",
        ["ndn-traffic-server", serverConfigurationFile],
        "-> Starting Traffic Server 4 (Prefix=ndn:/D/E, FreshnessPeriod=10ms, Data=E)")
    #time.sleep(0.1)

    time.sleep(2)
    p.killProcess("ndn-traffic-server-1")
    p.killProcess("ndn-traffic-server-2")
    p.killProcess("ndn-traffic-server-3")
    p.killProcess("ndn-traffic-server-4")
    time.sleep(2)

    checkServerStatus(p, "ndn-traffic-server-1", "Traffic Server 1")
    checkServerStatus(p, "ndn-traffic-server-2", "Traffic Server 2")
    checkServerStatus(p, "ndn-traffic-server-3", "Traffic Server 3")
    checkServerStatus(p, "ndn-traffic-server-4", "Traffic Server 4")

    #print p.getProcessOutput("ndn-traffic-server-1")
    #print p.getProcessOutput("ndn-traffic-server-2")
    #print p.getProcessOutput("ndn-traffic-server-3")
    #print p.getProcessOutput("ndn-traffic-server-4")

    p.killNfd()


if __name__ == "__main__":
    main()
